from sarban.methods import Methods

class SARBAN(Methods):
    def __init__(
        self,
        full_address: str,
        https: bool = True,
    ) -> None:
        super().__init__()

        self.full_address = full_address
        self.https = https
        self.session_string = None
        self.token = None



class Exceptions(Exception):
    def __init__(self, message, error_code):
        self.message = message
        self.error_code = error_code

class NotFound(Exceptions):
    code = 'NOT_FOUND'
    message = 'Error 404 has been received'

    def __init__(self):
        super().__init__(self.message, self.code)

    def __str__(self):
        return f"[{self.code}] {self.message}."

class BadLogin(Exceptions):
    code = 'BAD_LOGIN'
    message = 'Username or password is incorrect'

    def __init__(self):
        super().__init__(self.message, self.code)

    def __str__(self):
        return f"[{self.code}] {self.message}."

class AlreadyLogin(Exceptions):
    code = 'ALREADY_LOGIN'
    message = 'You are currently logged in'

    def __init__(self):
        super().__init__(self.message, self.code)

    def __str__(self):
        return f"[{self.code}] {self.message}."


from sarban.methods.base import Base
from sarban.methods.login import Login
from sarban.methods.inbounds import Inbounds
from sarban.methods.clients import Clients

class Methods(
    Base,
    Login,
    Inbounds,
    Clients
):
    pass



from typing import Union

import sarban
from sarban import errors

class Inbounds:
    def get_users(
        self: "sarban.SARBAN"
    ) -> Union[dict, errors.NotFound]:
        """Get inbounds of the Marzban.
        
        Returns:
            `~Dict | errors.NotFound`: On success, a dict is returned else 404 error will be raised
        """
        

        params = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.token}'
        }

        response = self.request(
            path="users",
            method="GET",
            headers=params
        )

        return self.verify_response(response)



import json
from typing import Any
import sarban
from sarban import errors

class Login:
    def login(
        self: "sarban.SARBAN",
        username: str,
        password: str
    ) -> Any:
        """Login into marzan panel.

        Parameters:
            username (``str``):
                Username of panel
                
            password (``str``):
                Password of panel

        Returns:
            `~Any`: On success, True is returned else an error will be raised
        """
        
        if self.session_string:
            raise errors.AlreadyLogin()
        


        headers = {
        'accept': 'application/json',
        'Content-Type': 'application/x-www-form-urlencoded'
        }

        data = {
            'grant_type': '',
            'username': username,
            'password': password,
            'scope': '',
            'client_id': '',
            'client_secret': ''
        }


        send_request = self.request(
            path="admin/token",
            method="POST",
            data=data,
            headers=headers
        )
        if send_request.status_code == 200:
            self.token = json.loads(send_request.text)["access_token"]
            return True
            
        raise errors.BadLogin()



import requests, json

import sarban
from sarban import errors

class Base:

    def request(
        self: "sarban.SARBAN",
        path: str,
        method: str,
        headers: dict,
        data: dict = None

    ) -> requests.Response:
        """Request to the Marzban.

        Parameters:
            path (``str``):
                The request path, you can see all of them in your API documention of MArzban
                
            method (``str``):
                The request method
                
            data (``dict``, optional):
                The request parameters, None is set for default but it's necessary for some POST methods
            
            headers (``dict``):
                The request headers

        Returns:
            `~requests.Response`: On success, the response is returned.
        """
        


        url = f"{self.full_address}/api/{path}"

        if method == "GET":
            response = requests.get(url, headers=headers, data=json.dumps(data), verify=self.https)
        elif method == "POST":
            if "admin/token" in path:
                response = requests.post(url, headers=headers, data=data, verify=self.https)
            else:
                response = requests.post(url, headers=headers, data=json.dumps(data), verify=self.https)
        elif method == "PUT":
            response = requests.put(url, headers=headers, data=json.dumps(data), verify=self.https)
        elif method == "DELETE":
            response = requests.delete(url, headers=headers, verify=self.https)



        return response

    def verify_response(
        self: "sarban.SARBAN",
        response: requests.Response
    ) -> requests.Response:
        if response.status_code != 404 and response.headers.get('Content-Type').startswith('application/json'):
            return response.json()
        
        raise errors.NotFound()


import json
from typing import Union
import time, uuid


import sarban
from sarban import errors

class Clients:
    def __UUIDgen() -> str:
        timestamp = int(time.time() * 1000)
        generated_uuid = str(uuid.uuid5(uuid.NAMESPACE_DNS, str(timestamp)))

        return generated_uuid

    def get_client(
        self: "sarban.SARBAN",
        username: str,
    ) -> Union[dict, errors.NotFound]:
        """Get client from the existing inbound.

        Parameters:
            username (``str``):
               username of the client

            
        Returns:
            `~Dict`: On success, a dict is returned or else 404 an error will be raised
        """
        
        # OLD WAY ==============================
        # all_users = self.get_users()

        # for user in range(all_users["total"]):
        #     if all_users["users"][user]["username"] == username:
        #         return all_users["users"][user]

        # raise errors.NotFound()
        #=======================================


        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.token}'
        }

        response = self.request(
            path=f"user/{username}",
            method="GET",
            headers=headers
        )

        return self.verify_response(response)

    def get_client_by_subLink(
        self: "sarban.SARBAN",
        token: str,
    ) -> Union[dict, errors.NotFound]:
        """Get client from the existing inbound.

        Parameters:
            token (``str``):
               token of the client(sublinks token)

            
        Returns:
            `~Dict`: On success, a dict is returned or else 404 an error will be raised
        """

        headers = {
            'accept': 'application/json',
        }

        response = self.request(
            path=f"{self.full_address}/sus/{token}/info",
            method="GET",
            headers=headers
        )

        return self.verify_response(response)

    def add_client(
        self: "sarban.SARBAN",
        username: str,
        inboundTag: list,
        note:str = "",
        enable: str = "active",
        flow: str = "xtls-rprx-vision",
        total_gb: int = 0,
        expire_time: int = 0,
    ) -> Union[dict, errors.NotFound]:
        """Add client to the existing inbounds.

        Parameters:
            username (``str``):
                username of client
                
            inboundTag (``str(list)``):
               list of inbounds avaible for client
                
            note (``str``, optional):
               Note for client
                
            enable (``str``, optional):
               Status of the client
                
            flow (``str``, optional):
               Flow of the client
                
            total_gb (``int``, optional):
                Download and uploader limition of the client and it's in GB
                
            expire_time (``str``, optional):
                Client expiration date and it's in timestamp
                
        Returns:
            `~Dict`: On success, a dict is returned else 404 error will be raised
        """
        
        
        data = {
        "username": username,
        "proxies": {
            "vless": {
            "flow": flow
            }
        },
        "inbounds": {
            "vless": inboundTag
        },
        "expire": expire_time,
        "data_limit": total_gb*1024**3,
        "data_limit_reset_strategy": "no_reset",
        "status": enable,
        "note": note
        }

        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.token}',
            'Content-Type': 'application/json'
        }


        response = self.request(
            path="user",
            method="POST",
            data=data,
            headers= headers
        )

        return self.verify_response(response)

    def delete_client(
        self: "sarban.SARBAN",
        username: str,
    ) -> Union[dict, errors.NotFound]:
        """Delete client

        Parameters:
            username (``str``):
               username of the client
                
        Returns:
            `~Dict`: On success, a dict is returned else 404 error will be raised
        """

        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.token}'
        }
        

        response = self.request(
            path=f"user/{username}",
            method="DELETE",
            headers=headers
        )

        return self.verify_response(response)

    def edit_client(
        self: "sarban.SARBAN",
        username: str,
        inboundTag: list,
        note:str = "",
        enable: str = "active",
        flow: str = "xtls-rprx-vision",
        total_gb: int = 0,
        expire_time: int = 0,
    ) -> Union[dict, errors.NotFound]:
        """edit client

        Parameters:
            username (``str``):
                username of client
                
            inboundTag (``str(list)``):
               list of inbounds avaible for client
                
            note (``str``, optional):
               Note for client
                
            enable (``str``, optional):
               Status of the client
                
            flow (``str``, optional):
               Flow of the client
                
            total_gb (``int``, optional):
                Download and uploader limition of the client and it's in GB
                
            expire_time (``str``, optional):
                Client expiration date and it's in timestamp
                
        Returns:
            `~Dict`: On success, a dict is returned else 404 error will be raised
        """


        headers = {
            'accept': 'application/json',
            'Authorization': f'Bearer {self.token}',
            'Content-Type': 'application/json'
        }
        data = {}


        # adding proxies
        new = {
            "proxies": {
                "vless": {
                    "flow": flow
                }
            }
        }
        data.update(new)
        
        # adding inbounds
        new = {
            "inbounds": {
                "vless": inboundTag
            }
        }
        data.update(new)

        # adding other deatails if needed
        if total_gb:
            new = {
                "data_limit": total_gb*(1024**3),
            }
            data.update(new)
        if enable != "active":
            new = {
                "status": enable
            }
            data.update(new)
        if expire_time:
            new = {
                "expire": expire_time
            }
            data.update(new)
        if note != "":
            new = {
                "note": note
            }
            data.update(new)


        response = self.request(
            path=f"user/{username}",
            method="PUT",
            headers=headers,
            data=data
        )

        return self.verify_response(response)


from setuptools import setup, find_packages

VERSION = "1.1.0"

with open("README.md", "r") as f:
    readme = f.read()

setup(
    name="sarban",
    version=VERSION,
    author="liwyd",
    description="marzban panel API easey-managing.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/liwyd/sarban",
    keywords=[
        "sarban",
        "marzban",
        "marzban python",
        "marzban panel"
    ],
    packages=find_packages(),
    install_requires=["requests"],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    license="MIT",
    python_requires='>=3.6'
)
